/*
 *            File: ColoriResource.java
 *          Autore: Roberto FULIGNI
 * Ultima modifica: 24/04/2023
 *
 *     Descrizione: Accesso alle API RESTful mediante
 *                  autenticazione HTTP BASIC oppure DIGEST.
 *                  Configurazione basata sul deployment descriptor web.xml
 *
 *       Requisiti: All'interno del container devono essere presenti:
 *
 *                     - i ruoli:
 *                          "gruppo1-rest"
 *                          "gruppo2-rest".
 *
 *                     - gli utenti:
 *                          "utente1-rest" (ruolo "gruppo1-rest")
 *                          "utente2-rest" (ruolo "gruppo2-rest")
 */

package edu.fauser.netlab;

import javax.ws.rs.*;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.UriInfo;
import java.net.URI;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

@Path("/colori")
@Consumes("application/json")
@Produces("application/json")
public class ColoriResource {
    @Context
    private UriInfo info;               // Contiene l'URI dell'API (utilizzata per generare alcune delle risposte)
    private static Map<Integer, Colore> colori = new TreeMap<Integer, Colore>();

    // Precaricamento di alcuni dati nella mappa di colori

    static {
        Integer id = Colore.getNuovoId();
        colori.put(id, new Colore(id, "rosso", "255,0,0", "#ff0000"));
        id = Colore.getNuovoId();
        colori.put(id, new Colore(id, "verde", "0,255,0", "#00ff00"));
    }

    // Acquisizione della lista di colori presenti nella mappa.
    // Ogni colore è descritto brevemente mediante gli attributi id e nome
    //
    // curl -v -u utente1-rest:123456 http://localhost:8080/server-colori-http/api/colori

    @GET
    public Response getColori() {
        List<ColoreBreve> lista = new ArrayList<ColoreBreve>();
        for (Map.Entry<Integer, Colore> entry : colori.entrySet()) {
            Integer id = entry.getKey();
            String nome = entry.getValue().getNome();
            lista.add(new ColoreBreve(id, nome));
        }

        return Response
                .ok(lista)
                .build();
    }

    // Inserimento di un nuovo colore
    //
    // curl -v -u utente1-rest:123456 -X POST -H "Content-Type: application/json" ^
    // http://localhost:8080/server-colori-http/api/colori ^
    // -d "{\"nome\":\"blu\", \"rgb\": \"0,0,255\", \"hex\": \"#0000ff\"}"

    @POST
    public Response nuovoColore(Colore dati) {
        String nome = dati.getNome();
        String rgb = dati.getRgb();
        String hex = dati.getHex();
        if (nome == null || rgb == null || hex == null) {
            return Response
                    .status(Response.Status.BAD_REQUEST)
                    .entity(new Errore("Descrizione incompleta del colore"))
                    .build();
        }
        Integer id = Colore.getNuovoId();

        colori.put(id, new Colore(id, nome, rgb, hex));
        URI nuovoUri = info.getRequestUriBuilder().path(id.toString()).build();
        return Response
                .created(nuovoUri)
                .build();
    }

    // Acquisizione della descrizione completa di un singolo colore.
    // Il colore è identificato dal parametro id.
    //
    // curl -v -u utente1-rest:123456 -i http://localhost:8080/server-colori-http/api/colori/1

    @GET
    @Path("/{id}")
    public Response getColore(@PathParam("id") Integer id) {

        Colore colore = colori.get(id);
        if (colore == null) {
            return Response
                    .status(Response.Status.NOT_FOUND)
                    .entity(new Errore("Colore inesistente"))
                    .build();
        }

        return Response
                .ok(colore)
                .build();
    }

    // Aggiornamento completo di un singolo colore all'interno della mappa.
    // Il colore è identificato dal parametro id.
    //
    // curl -v -u utente1-rest:123456  -X PUT -H "Content-Type: application/json"  ^
    // http://localhost:8080/server-colori-http/api/colori/1 ^
    // -d "{\"nome\":\"rosso scuro\", \"rgb\":\"185,0,0\", \"hex\": \"#b90000\"}"

    @PUT
    @Path("/{id}")
    public Response setColore(@PathParam("id") Integer id, Colore dati) {
        Colore c = colori.get(id);
        if (c == null) {
            return Response
                    .status(Response.Status.NOT_FOUND)
                    .entity(new Errore("Colore inesistente"))
                    .build();
        }

        String nome = dati.getNome();
        String rgb = dati.getRgb();
        String hex = dati.getHex();

        if (nome == null || rgb == null || hex == null) {
            return Response
                    .status(Response.Status.BAD_REQUEST)
                    .entity(new Errore("Descrizione incompleta del colore"))
                    .build();
        }
        c.setNome(nome);
        c.setRgb(rgb);
        c.setHex(hex);

        return Response
                .ok()
                .build();
    }

    // Aggiornamento parziale di un singolo colore all'interno della mappa.
    // Il colore è identificato dal parametro id.
    //
    // curl -v -u utente1-rest:123456  -X PATCH -H "Content-Type: application/json"  ^
    // http://localhost:8080/server-colori-http/api/colori/1 -d "{\"rgb\":\"255,1,1\"}"

    @PATCH
    @Path("/{id}")
    public Response aggiornaColore(@PathParam("id") Integer id, Colore dati) {
        Colore c = colori.get(id);
        if (c == null) {
            return Response
                    .status(Response.Status.NOT_FOUND)
                    .entity(new Errore("Colore inesistente"))
                    .build();
        }

        String nome = dati.getNome();
        if (nome != null)
            c.setNome(nome);

        String rgb = dati.getRgb();
        if (rgb != null)
            c.setRgb(rgb);

        String hex = dati.getHex();
        if (hex != null)
            c.setHex(hex);

        return Response
                .ok()
                .build();
    }

    // Eliminazione di un singolo colore dalla mappa.
    // Il colore è identificato dal parametro id.
    //
    // curl -v -u utente1-rest:123456  -X DELETE http://localhost:8080/server-colori-http/api/colori/1
    @DELETE
    @Path("/{id}")
    public Response eliminaColore(@PathParam("id") Integer id) {
        if (!colori.containsKey(id)) {
            return Response
                    .status(Response.Status.NOT_FOUND)
                    .entity(new Errore("Colore inesistente"))
                    .build();
        }

        colori.remove(id);

        return Response
                .ok()
                .build();
    }
}
